      Program Site_Amp_Ratios

* Read files made using site_amp_batch and compute ratios of amplifications
 
*And here is a sample control file:
*!Control file for site_amp_ratios
*!Name of summary file:
*   site_amp_ratios.sum
*!Name of output file:
*   site_amp_ratios.col
*!Name of reference file:
*  willow_average_slowness.mdl.vtt.site_amp_batch.asc
*!List of input files
* wils_rvelslant2.ss.vtt.site_amp_batch.asc
* sc133_t1.ss.vtt.site_amp_batch.asc
* willow_3a_s.ss.vtt.site_amp_batch.asc
*Stop
 
* Dates:  08/24/07 - Written by D.M. Boore, based on site_amp
*         09/10/07 - nskip = 4 in two places (reference file and 
*                    the other file) before reading nfreq_in, f4model_bottom_in(nin)
*                    because added a text header line to 
*                    site_amp_batch output (so that its output has
*                    the same number of header lines as site_amp).
!         08/23/12 - Allow for new headers in site_amp output
!                  - Add names of file to headers in output file
!         08/24/12 - Replace "freqlow" with "freq4model_bottom"
!         08/04/17 - Account for new site_amp_batch_output

      real freq_ref(4000), freq_in(4000)
      real amp_ref(4000), amp_in(4000,20), amp_rat(4000,20)

      real depth(10000), vel(10000), dens(10000), qi(10000),
     :     akappa(10000), slope_v(10000), slope_r(10000), 
     :     slope_qi(10000), tt(10000), d2b(0:10000), 
     :     depth_in(10000), vel_in(10000), dens_in(10000), 
     :     q_in(10000)

      real freq_spcfy(10000), tt_spcfy(0:10000), z_spcfy(0:10000),
     :     avgvel_spcfy(10000), avgdens_spcfy(0:10000), 
     :     akappa_spcfy(0:10000), avgaoi_spcfy(10000), 
     :     ampli_spcfy(10000)
     
      real f4model_bottom_in(20)

      real dum(20)
      
      real avgvel(10000), avgdens(10000), avgaoi(10000)

      integer iout(10000), nc_colhead(42), nc_f_in(20)
      
      parameter(ny_max = 20, ncolhead_max = 42)
      parameter(nper_max = 300)
      
      character colhead(ncolhead_max)*10, 
     :          xref_c*10, x_c(ny_max)*10,
     :          yref_c*10, y1_c(ny_max)*10, y2_c(ny_max)*10,
     :          colstem*6

      character f_ctl*200, f_sum*200, f_in(20)*200, f_out*200,
     :          f_ref*200,
     :          constant_vel*1, dtbot*1, ext*60,
     :          buf*80, buf4*4,
     :          answer*1, tempbuf*8, 
     :          bffr*80, rho_col_exist*3, use_rho_col*3
      character col_d*10, col_v*10, col_r*10, col_qi*10, 
     :          col_f*10, col_a*10, col_k*10, col_ak*10,
     :          cbuf(16)*10, spcfy_freq*1, units_c*10
  
      character cmnts2skip(50)*100, type_of_depth*10

      logical susp_log, dens_from_model, q_from_model,
     :        f_mdl_exist, f_exist, specify_freqs,
     :        log_spaced_freqs
      logical specify_dens_coeff

      pi = 4.0 * atan(1.0)
      d2r = pi/180.0

*Get name of control file and open the file:

      write(*,*)

      f_exist = .false.
      do while (.not. f_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :      '   Enter name of control file (cr=site_amp_ratios.ctl; '//
     :         'ctl-brk to quit): '
        read(*, '(a)') f_ctl
        if (f_ctl(1:4) .eq. '    ') f_ctl='site_amp_ratios.ctl'
        call trim_c(f_ctl, nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_exist)
        if (.not. f_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl, file=f_ctl(1:nc_f_ctl), status='unknown')

*Get summary file name:
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call trim_c(f_sum, nc_f_sum)
      call get_lun(nu_sum)
      open(unit=nu_sum,file=f_sum(1:nc_f_sum),status='unknown')
 
*Get output file name:
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      f_out = ' '
      read(nu_ctl,'(a)') f_out
      call trim_c(f_out, nc_f_out)
      call get_lun(nu_out)
      open(unit=nu_out,file=f_out(1:nc_f_out),status='unknown')
 
*Get reference file name:
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      f_ref = ' '
      read(nu_ctl,'(a)') f_ref
      call trim_c(f_ref, nc_f_ref)
      call get_lun(nu_ref)
      open(unit=nu_ref,file=f_ref(1:nc_f_ref),status='unknown')
      
      nskip = 5
      call skip(nu_ref, nskip)
      
      read(nu_ref,'(t34,i4, t146,es10.3)') nfreq_ref, f4model_bottom_ref
      write(nu_sum, '(a,1x,i5)') ' nfreq_ref = ', nfreq_ref
      
      nskip = 2
      call skip(nu_ref, nskip)
      
      do j = 1, nfreq_ref
      
        read(nu_ref,*) (dum(i),i=1,18)
        freq_ref(j) = dum(15)
        amp_ref(j) = dum(18)
*DEBUG
      print *,' nfreq, freq, amp = ', 
     :      j, freq_ref(j), amp_ref(j)
*DEBUG

      end do
 
      close(nu_ref)
      
* Read and process files:
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      
       
      nin = 0

1000  continue

      buf = ' '
      read(nu_ctl,'(a)') buf
      call trim_c(buf, nc_buf)
      buf4 = ' '
      buf4 = buf(1:4)
      call upstr(buf4)
      if (buf4 .eq. 'STOP') then
      
        close(nu_ctl)
      
        go to 2000

      end if     

      f_in(nin+1) = ' '
      f_in(nin+1) = buf(1:nc_buf)
      call trim_c(f_in(nin+1), nc_f_in(nin+1))
* DEBUG
*      write(nu_debug,'(a)') ' f_in = '//f_in(nin+1)(1:nc_f_in)
* DEBUG
      f_exist = .false.
      inquire(file=f_in(nin+1)(1:nc_f_in(nin+1)), exist=f_exist)
      if (.not. f_exist) then
         write(*,'(a)') 
     :  ' ******* FILE '//f_in(nin+1)(1:nc_f_in(nin+1))//
     :  ' DOES NOT EXIST ******* '
         write(*,'(a)') ' Skipping to next file'
         go to 1000
      end if
      
      nin = nin + 1
      
      call get_lun(nu_in)
      open(unit=nu_in,file=f_in(nin)(1:nc_f_in(nin)),
     :     status='unknown')
      
* DEBUG
      write(nu_sum,*)  
      write(nu_sum,'(a)')
     :          ' processing file '//f_in(nin)(1:nc_f_in(nin))
      write(nu_sum,'(a)') 
     :   ' nfreq, nin, freq, amp_in, amp_ref, amp_rat = '
* DEBUG
      
      nskip = 5
      call skip(nu_in, nskip)
      
      read(nu_in,'(t34,i4, t146,e10.3)') 
     :      nfreq_in, f4model_bottom_in(nin)
      write(nu_sum, '(a,1x,i2, 1x,i5, 1p, 1x,e10.3)') 
     :    ' nin, nfreq_in, f4model_bottom_in = ', 
     :      nin, nfreq_in, f4model_bottom_in(nin)
      
      nskip = 2
      call skip(nu_in, nskip)
      
      do j = 1, nfreq_in
      
        read(nu_in,*) (dum(i),i=1,18)
      
        freq_in(j) = dum(15)
        amp_in(j, nin) = dum(18)
      
        if( freq_in(j) .ne. freq_ref(j)) then
          print *,' error, mismatch of freqs; going to next file.'
          write(nu_sum,'(a)') 
     :        ' error, mismatch of freqs; going to next file.'
         write(nu_sum,'(1x,a, 1x,i3, 1p,2(1x,e10.3))') 
     :      ' nfreq_in, freq_in, freq_ref = ', 
     :        j, freq_in(j), freq_ref(j) 
         close(nu_in)
         nin = nin - 1
         go to 1000
       end if
      
      
      
      amp_rat(j,nin)=amp_in(j,nin)/amp_ref(j) 
*DEBUG
        write(nu_sum,'(1x,i3,1x,i2, 1p, 4(1x,e10.3))')
     :  j, nin, freq_ref(j), amp_in(j, nin),
     :  amp_ref(j), amp_rat(j, nin) 
*DEBUG
      end do
      
*DEBUG
      print *,' Finished with file: '//f_in(nin)(1:nc_f_in(nin))
*DEBUG
      
      go to 1000
        
2000  continue

* Write output

      colhead = ' '
      
      colhead(1) = 'freq'

      colstem = ' '
      colstem(7-min(nc_f_ref,6):6) = f_ref(1:min(nc_f_ref,6))

      colhead(2) = colstem//'_amp'

      do j = 1, nin
      
        colstem = ' '
        colstem(7-min(nc_f_in(j),6):6) = f_in(j)(1:min(nc_f_in(j),6))

        colhead(2+2*j-1) = f_in(j)(1:6)//'_amp'
        colhead(2+2*j) = f_in(j)(1:6)//'_rat'
          
      end do
      
      do j = 1, 2+2*nin
      
        call trim_c(colhead(j), nc_colhead(j))
        
      end do
      
      write(nu_out,'(a)')   '!Name of denominator file: '
      write(nu_out,'(a)')   '!   '//f_ref(1:nc_f_ref)
     
      write(nu_out,'(a)')   '!Name of numerator files: '
      do j = 1, nin
        write(nu_out,'(a)') '!   '//f_in(j)(1:nc_f_in(j))
      end do
     
     
      write(nu_out,'(7x,a4, 41(1x,a10))') (colhead(j),j=1,2+2*nin)
      
      

      do i = 1, nfreq_ref
      
        if (freq_ref(i) .lt. f4model_bottom_ref) then
           go to 3000
        end if
      
        xref_c = ' '
        yref_c = ' '
        x_c = ' '
        y1_c = ' '
        y2_c = ' '
        
        write(xref_c, '(1p,e10.3)') freq_ref(i)
        write(yref_c, '(1p,e10.3)') amp_ref(i)
         
        do j = 1, nin
        
          if (freq_in(i) .ge. f4model_bottom_in(j)) then
            write(y1_c(j), '(1p,e10.3)') amp_in(i,j) 
            write(y2_c(j), '(1p,e10.3)') amp_rat(i,j) 
          end if
       
        end do
       
       write(nu_out,'(42(1x,a10))') 
     :     xref_c, yref_c, (y1_c(j), y2_c(j), j = 1, nin)
     
      end do  ! loop back for another frequency
      
3000  continue       
        
      close(nu_sum)
      close(nu_out)
 
      stop
      end
      
      
      include 'site_amp_util_subs.for'
     