
         Program SMC_Apply_Site_Response

! This program computes the time domain response of an input motion after
! applying site amplifications from the program Nrattle.
! Note that it is crucial that the Nrattle parameters fhigh and nfreq are 
! consistent with the sampling rate (sps) and length (npts) of the input time series.  In particular:
!
!         fhigh= sps/2 (the Nyquist frequency) (where sps = 1/dt)
!
!         nfreq = npts/2 + 1
!
! Another constraint is that npts be a power of 2.   The length of the input time series will
! probably not be a power of 2, but that can be changed by using smc_snip, adjusting the 
! parameters tskip and tlength so that the snipped time series is a power of 2. The tlength parameter
! might need to be slightly larger than the desired length in order to overcome roundoff
! in reading the smc_snip control file parameters; check the headers of the output file to make sure that npts is
! the desired value.
!
! Another point:  the smc_apply_site response program assumes that the nrattle output was created using a version 
! of nrattle with a date of 12 December 2015 or later (because a line giving the number of layers
! in the model was added before the model, in the file containing the complex amplification values).
!

! For convenience, here are the inverses of the relations above:
!
!         sps = 2*fhigh
!         npts = 2*(nfreq - 1)

!!Control file for program SMC_Apply_Site_Response
!! As many comment lines as desired, each starting with "!"
!! The string "pp:" indicates a new set of processing parameters
!! to be applied to the following smc files.  The parameters are given on the
!! following lines.
!! Revision of program involving a change in the control file on this date:
!   12/21/15
!!Name of summary file:
! smc_apply_site_response.sum
!!Name of complex transfer function, made with nrattle.
! composite_model.mdl.aoi_00.for_ts.q50.nrattle_complex_spectra.out
!!time shift for impulse response [in freq dom, multiply FS by exp(+i*omega*tshift)]:
! 1.0
!!String to append to input file name when constructing the output filename.
! .site_response.smc
!!mean info to apply to input file:
!! type of mean (0=no correction, 1=standard average; -1=specify mean)
!! t4mean_strt, t4mean_stop, mean (use if type = -1)
!! NOTE: need entries even if the parameter is not used (because read
!! info use list-directed [unformatted] input) 
!! Note that if working with a processed record that includes the 
!! complete time series produced by the processing (e.g., the padded time series
!! for acausal filtering or the post-processing baseline corrected time series
!! produced from pad-stripped or pass-through data in the first NGA project), 
!! it is inadvisible to remove the mean as this may 
!! produce unintended distortions at long periods; in this case set t4mean < 0.0.
!! If the record is pad-stripped, however, as provided by most
!! data centers, without post-processing to remove drifts in displacement,
!! a mean determined from the overall record probably should be 
!! removed.  But in this case the displacement will probably be inaccurate
!! (compared to the displacement derived from the padded, filtered acceleration
!! time series), it may be that a better approach is to specify the initial
!! velocity and displacement of the record---I am working on that (11/26/10):
! 0  1.0 20.0  -999.
!!List of files 
! t0781pea.smc
! t0781ped.smc
!Stop

! Dates: 12/21/15 - Written by d. boore, based on c:\svel\i10_sat\decon_ts.for, 
!                   c:\site_amp\programs\impulse_response_from_nrattle_output.for,
!                   \forprogs\smc2vd.for

      real a(:), s(:), freq(:), areal_nrattle(:),aimag_nrattle(:) 
      allocatable :: a, s, freq, areal_nrattle,aimag_nrattle 

      real real_head(50) 

      Complex CARG,CEXP,CTEMP, FS(:), FS_in(:), FS_nrattle(:) 
      allocatable fs, fs_in, fs_nrattle
      
      character comments(500)*80, f_a*200,
     : f_sum*200, f_fs*200, f_out*200, buf*500, buf4*4,
     : smc_precision*20, extension_c*100
     
      character  f_ctl*200, cmnts2skip(60)*100 

      character date_ctl_correct*8, date_ctl_in*30
      logical  file_exist, f_a_exist 
      
      character char_head(11)*80
      integer int_head(48), NPW2, status 

      file_exist = .false.
      
      twopi = 8.0 * atan(1.0)

      DO
        f_ctl = ' '
        write(*, '(a)') 
     :    ' Enter name of control file '//
     :    '("Enter" = smc_apply_site_response.ctl;'//
     :    ' ctl-brk to quit): '
        read(*, '(a)') f_ctl
        call trim_c(f_ctl, nc_f_ctl)
        if (f_ctl(1:4) .eq. '    ') 
     :        f_ctl = 'smc_apply_site_response.ctl'
        call trim_c(f_ctl, nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=file_exist)
        IF (file_exist) EXIT
        write(*,'(a)') ' ******* FILE '//f_ctl(1:nc_f_ctl)//
     :                 ' DOES NOT EXIST, '//
     :                 'ENTER A VALID FILE NAME ******* '
      END DO
      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),
     :     status='unknown')

!Check version of control file
      date_ctl_correct = ' '
      date_ctl_correct = '12/21/15'
      call trim_c(date_ctl_correct,nc_date_ctl_correct)

      call skipcmnt(nu_ctl, cmnts2skip,nc_cmnts2skip)
      date_ctl_in = ' '
      read(nu_ctl,'(a)') date_ctl_in
      call trim_c(date_ctl_in,nc_date_ctl_in)
      
      if (date_ctl_correct(1:nc_date_ctl_correct) /=
     :    date_ctl_in(1:nc_date_ctl_in)) then
        write(*,'(a)') 
     :     ' The control file has the wrong date ('//
     :       date_ctl_in(1:nc_date_ctl_in)//'); '//
     :      'the proper date is '//date_ctl_correct
        write(*,'(a)') 
     :     ' STOP!'
        close(nu_ctl)
        stop
      end if

      call skipcmnt(nu_ctl, cmnts2skip,nc_cmnts2skip)
      f_sum = ' '
      read(nu_ctl, '(a)') f_sum
      call trim_c(f_sum, nc_f_sum)
      call get_lun(nu_sum)
      open(unit=nu_sum,file=f_sum(1:nc_f_sum),
     :     status='unknown')

      write(nu_sum,'(a)') ' Output of program SMC_Apply_Site_Response:'
      write(nu_sum,'(a)') '   Control file = '//f_ctl(1:nc_f_ctl)


!Get file name with complex output

      call skipcmnt(nu_ctl, cmnts2skip,nc_cmnts2skip)
      f_fs = ' '
      read(nu_ctl, '(a)') f_fs
      call trim_c(f_fs, nc_f_fs)
      call get_lun(nu_fs)
      open(unit=nu_fs,file=f_fs(1:nc_f_fs),
     :     status='unknown')

!Get tshift

      call skipcmnt(nu_ctl, cmnts2skip,nc_cmnts2skip)
      read(nu_ctl, *) tshift
      write(nu_sum,'(1x,a, 1x,es11.4)') ' tshift = ', tshift
      write(nu_sum,'(1x,a, 1x,es11.4)') ' twopi = ', twopi

!Get extension to add to input file name to create the output file name

      call skipcmnt(nu_ctl, cmnts2skip,nc_cmnts2skip)
      extension_c = ' '
      read(nu_ctl,'(a)') extension_c
      call trim_c(extension_c, nc_extension_c)
      write(nu_sum,'(a)') 
     :  ' extension_c ='//extension_c(1:nc_extension_c)
 
!Extract nfreq, fhigh, nlayers:

      call skip(nu_fs,5)
      buf = ' '
      read(nu_fs,'(a)') buf
      call trim_c(buf, nc_buf)
      read(buf(index(buf(1:nc_buf),'=')+1:nc_buf),*) nfreq
      write(nu_sum,'(1x,a, 1x,i6)') ' nfreq = ', nfreq
      
      buf = ' '
      read(nu_fs,'(a)') buf
      call trim_c(buf, nc_buf)
      read(buf(index(buf(1:nc_buf),'=')+1:nc_buf),*) fhigh
      write(nu_sum,'(1x,a, 1x,es11.4)') ' fhigh = ', fhigh
      
      call skip(nu_fs,4)
      buf = ' '
      read(nu_fs,'(a)') buf
      call trim_c(buf, nc_buf)
      read(buf(index(buf(1:nc_buf),'=')+1:nc_buf),*) nlayers
      write(nu_sum,'(1x,a, 1x,i6)') ' nlayers = ', nlayers
       
!Compute npw2 and make sure that it is a power of 2.
      npts = 2*(nfreq - 1)
      ln2_npts = int(alog(real(npts))/alog(2.0))
      write(nu_sum,'(1x,a, 1x,i6, 1x,i6)') 
     :                      ' npts, ln2_npts = ', npts, ln2_npts
      if ( 2.0**ln2_npts /= npts) then
        write(*, '(1x,a, 1x,i6, 1x,a)') 
     :    ' npts = ', npts, ' not a power of 2; QUITTING'
        stop
      else
        npw2 = npts
      end if

      allocate( freq(npw2+10), 
     :     areal_nrattle(npw2+10),aimag_nrattle(npw2+10),
     :     fs_nrattle(npw2+10) )
      
! Fill nrattle array with zeros
      do i = 1, npw2
        FS_nrattle(i) = cmplx(0.0,0.0)
      end do

! Read in NRATTLE transfer function

      nskip = 2 + nlayers + 8
      call skip(nu_fs, nskip) !should be first line of complex spectrum
         
      do i = 1, npw2/2+1
        read(nu_fs,*) idum, areal_nrattle(i), aimag_nrattle(i), freq(i)
        if (i == 1) then
          write(nu_sum,'(1x,a, 1x, es11.4)') 
     :       ' For '//f_fs(1:nc_f_fs)//', freq(1) = ', freq(1)
        end if
        if (i == npw2/2 + 1) then
          write(nu_sum,'(1x,a, 
     :                   1x,i6, 
     :                   1x,a, 1x,es11.4)')
     :      ' For '//f_fs(1:nc_f_fs)//', freq(',
     :           npw2/2+1,
     :                     ') = ', freq(npw2/2+1)
        end if 
      end do

! Change NRATTLE transfer function to Complex and apply shift operator
! Note the conjg operator was used to correct the output being time reversed.
! I did a number of runs, trying different isign for FORK, etc, before 
! finding that this worked.  I am not sure why, but it probably has to do with the
! sign assumptions in NRATTLE.
      do i =1, npw2/2+1
       FS_nrattle(i)=cexp(cmplx(0.0,twopi*freq(i)*tshift))*
     :        conjg(CMPLX(areal_nrattle(i),aimag_nrattle(i)))
      end do

!########################## Work with input time series ########################

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      read(nu_ctl,*) 
     :   itype_mean, t4mean_strt_in, t4mean_stop_in, spec_mean
      write(nu_sum,'(a)') 
     :  ' itype_mean, t4mean_strt_in, t4mean_stop_in, spec_mean:'
      write(nu_sum,*) 
     :    itype_mean, t4mean_strt_in, t4mean_stop_in, spec_mean

      num = 0
      
!Loop over input time series
      loop over files: DO
      buf = ' '
      read(nu_ctl,'(a)',iostat=status) buf
      if(status /= 0) then
        EXIT loop over files
      end if
      call trim_c(buf,nc_buf)
      buf4 = ' '
      buf4 = buf(1:4)
      call upstr(buf4)
      if(nc_buf == 0 .or. buf(1:1) == '!') then
        CYCLE loop over files
      end if
      if(buf4(1:4) == 'STOP') then
        EXIT loop over files
      end if
      
      f_a = ' '
      f_a = buf(1:nc_buf)
      nc_f_a = nc_buf 
      
      f_a_exist = .false.
      inquire(file=f_a(1:nc_f_a), exist=f_a_exist)

      if (.not. f_a_exist) then
          write(*,'(a)') ' ****** FILE '//f_a(1:nc_f_a)// 
     :          ' DOES NOT EXIST; SKIPPING!!! ******* '
          CYCLE loop over files
      end if

! Passed tests; proceed

      num = num + 1

! Get number of points on this loop:

      call smc_npts(f_a(1:nc_f_a), npts, sps)

! Test if npts = npw2 (as required for compatibility of the site response and the input acceleration)

      if (npts /= npw2) then
        write(*,'(1x,a, 1x,i6, 1x,i6)') 
     :       ' ERROR: Program cycling to next time series'//
     :       ' because npts and npw2 '//
     :        'not equal to one another'
        write(*,'(1x,a, 1x,i6, 1x,i6)') 
     :       ' npts, npw2 = ', npts, npw2
        write(nu_sum,'(1x,a, 1x,i6, 1x,i6)') 
     :       ' ERROR: Program cycling to next time series'//
     :       ' because npts and npw2 '//
     :        'not equal to one another'
        write(nu_sum,'(1x,a, 1x,i6, 1x,i6)') 
     :       ' npts, npw2 = ', npts, npw2
        cycle  ! return for the next time series
      end if

! Test if sps = 2*fhigh (as required for compatibility of the site response and the input acceleration)

      if (sps /= 2.0*fhigh) then
        write(*,'(1x,a)') 
     :       ' ERROR: Program cycling to next time series'//
     :       ' because sps /= 2*fhigh '//
     :        'not equal to one another'
        write(*,'(1x,a, 1x,es12.5, 1x,es12.5)') 
     :       ' sps, 2*fhigh = ', sps, 2.0*fhigh
        write(nu_sum,'(1x,a)') 
     :       ' ERROR: Program cycling to next time series'//
     :       ' because sps /= 2*fhigh '//
     :        'not equal to one another'
        write(nu_sum,'(1x,a, 1x,es12.5, 1x,es12.5)') 
     :       ' sps, 2*fhigh = ', sps, 2.0*fhigh
        cycle  ! return for the next time series
      end if

! Now read the data from the files:

      write(*,'(1x,a)') 
     :   'Processing file:', f_a(1:nc_f_a)

      tskip = 0.0
      tlength = -1.0

      allocate( a(npw2+10), s(npw2+10),
     :     fs(npw2+10), fs_in(npw2+10) )

      call SMCRead(f_a(1:nc_f_a),  
     :               tskip, tlength,
     :               a,  
     :               char_head, int_head, real_head, comments)
     
 
      sps = real_head(2)
      ncomments_a = int_head(16)
      npts = int_head(17)
      dt = 1.0/sps
      tend = float(npts-1)*dt

! Step 1: Remove a mean from the whole record

      t4mean_strt = t4mean_strt_in
      t4mean_stop = t4mean_stop_in
        
!  Determine the appropriate mean:
      if (itype_mean == 0) then
        n_t4mean = 0
        avg = 0.0
      else if (itype_mean == -1) then   ! use specified mean
        avg = spec_mean
      else if (itype_mean == 1) then
        if (t4mean_strt < 0.0) t4mean_strt = 0.0
        if (t4mean_stop > tend) t4mean_stop = tend
        n_t4mean_strt = nint(sps * t4mean_strt + 1.1)
        n_t4mean_stop = nint(sps * t4mean_stop + 1.1)
        n_t4mean = n_t4mean_stop - n_t4mean_strt + 1
        t4mean = float(n_t4mean -1)/sps
!        write(nu_sum,'(a,1x,f6.2)') ' t4mean = ', t4mean
        cum_avg = 0.0d0
        do i = n_t4mean_strt, n_t4mean_stop
          cum_avg = cum_avg + dble(a(i))
        end do
        avg = sngl( cum_avg/dble( real(n_t4mean) ) )
      else            ! shouldn't be here!
        write(nu_sum, '(a,1x,i3,1x,a)') '  Should not be here, '//'
     :       improper value of itype_mean.  itype_mean = ', 
     :       itype_mean, ' QUITTING!!'
        write(*) '  Should not be here, '//'
     :       improper value of itype_mean.  itype_mean = ', 
     :       itype_mean, ' QUITTING!!'
        deallocate (a, s, fs, fs_in )
        EXIT loop over files
      end if

!  Remove the mean from the whole record:
      do i = 1, npts
        a(i) = a(i) - avg
      end do

!Compute FS of input record:

! Fill S1 array with zeros
       do i = 1, NPW2
         fs_in(i) = cmplx(0.0,0.0)
       end do
! Change Input Accelerogram to COMPLEX
       n_shift=0
       do i=1, NPW2/2
         fs_in(i+n_shift)=cmplx(a(i), 0.0)
       end do 
! Input record to frequency domain      
       call fork(NPW2,fs_in,1.0) 


!Multiply by the nrattle response and transform back to the time domain:

       fs = cmplx(0.0,0.0)
       
       do i = 1, npw2/2 + 1
         fs(i) = fs_nrattle(i) * fs_in(i)
       end do

! Fill array from Nyquist to NPW2
       do i = NPW2/2 +2, NPW2
         FS(i) = CONJG(FS(NPW2+2-i))
       end do

! Transform back to time domain
      call fork(NPW2,FS,-1.0)   
!DEBUG
      write(nu_sum,'(1x,a)') 'index, real(FS), imag(FS)'
      do i = 1, npw2
        write(nu_sum,'(1x,i6, 1x,es11.4, 1x,es11.4)') 
     :      i, real(FS(i)), aimag(FS(i))
      end do
!DEBUG

      sps = 2.0*fhigh  

!      f2t_factor = sps/sqrt(real(npw2))  ! not needed if go from TD -> FD -> TD
      f2t_factor = 1.0
      do i=1,NPW2
        S(i)=f2t_factor*real(FS(i))
      end do

      int_null = -32768
      do i = 1, 48
        int_head(i) = int_null
      end do
 
      real_null = 1.7e+38
      do i = 1, 50
        real_head(i) = real_null
      end do

      do i = 2, 11
        char_head(i) = '*'
      end do


      ncomments_a = 0
      ncomments = ncomments_a + 1
      comments(ncomments) = '|'
      ncomments = ncomments + 1
      comments(ncomments)= 
     :      '| Output of program SMC_Apply_Site_Response'
      ncomments = ncomments + 1
      comments(ncomments)= '| File with complex spectrum from '//
     :                           'Nrattle:  '
      ncomments = ncomments + 1
      comments(ncomments)= '| '//f_fs(1:nc_f_fs)
      ncomments = ncomments + 1
      comments(ncomments)= '| File being processed ' 
      ncomments = ncomments + 1
      comments(ncomments)= '| '//f_a(1:nc_f_a)

      int_head(16)=ncomments

      int_head(17) = npw2

      real_head(2) = sps
      itype = 0
      smc_precision = ' '
      smc_precision = 'HIGH'
      call trim_c(smc_precision, nc_smc_precision)

      f_out = ' '
      f_out = f_a(1:nc_f_a)//extension_c(1:nc_extension_c)
      call trim_c(f_out, nc_f_out)
      write(nu_sum, '(1x,a, 1x,es11.4)')
     :      ' For '//f_out(1:nc_f_out)//', sps = ', sps

        write(*,'(1x,a)') 
     :   'Writing acceleration output to file:', f_out(1:nc_f_out)

      call SMCwrite(f_out(1:nc_f_out),
     :                smc_precision(1:nc_smc_precision),
     :                S, char_head, itype, 
     :                int_head, real_head,comments)

 
      deallocate (a, s, fs, fs_in)

      END DO loop over files

! Close control and output files:

      deallocate (fs_nrattle)
      close(nu_ctl)
      close(nu_sum)

      Stop
      End

      include '\forprogs\distaz.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\skip.for'
      include '\forprogs\mnmaxidx.for'
      include '\forprogs\smcread.for'
      include '\forprogs\smcwrite.for'
      include '\forprogs\fork.for'
      include '\forprogs\skipcmnt.for'
      include '\forprogs\smc_npts.for'
      include '\forprogs\upstr.for'
      



      
 